@@ -10,6 +10,10 @@ from di import BaseContainer
 from starlette.responses import Response
 
 
-class xpressoASGIExtension(TypedDict):
+class _XpressoASGIExtension(TypedDict):
     container: BaseContainer
+    response_sent: bool
+
+
+class XpressoASGIExtension(_XpressoASGIExtension, total=False):
     response: Response
